'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _es6Promise = require('es6-promise');

var _es6Promise2 = _interopRequireDefault(_es6Promise);

var _isImage = require('is-image');

var _isImage2 = _interopRequireDefault(_isImage);

var _isUrl = require('is-url');

var _isUrl2 = _interopRequireDefault(_isUrl);

var _slateDevLogger = require('slate-dev-logger');

var _slateDevLogger2 = _interopRequireDefault(_slateDevLogger);

var _loadImageFile = require('./load-image-file');

var _loadImageFile2 = _interopRequireDefault(_loadImageFile);

var _path = require('path');

var _slateReact = require('slate-react');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Insert images on drop or paste.
 *
 * @param {Object} options
 *   @property {Function} insertImage
 *   @property {Array} extensions (optional)
 * @return {Object} plugin
 */

function DropOrPasteImages() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var insertImage = options.insertImage,
      extensions = options.extensions;


  if (options.applyTransform) {
    _slateDevLogger2.default.deprecate('0.6.0', 'The `applyTransform` argument to `slate-drop-or-paste-images` has been renamed to `insertImage` instead.');
    insertImage = options.applyTransform;
  }

  if (!insertImage) {
    throw new Error('You must supply an `insertImage` function.');
  }

  /**
   * Check file extension against user-defined options.
   *
   * @param {Type} string
   * @return {Boolean}
   */

  function matchExt(type) {
    var accepted = false;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = extensions[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var ext = _step.value;

        if (type.includes(ext)) accepted = true;
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return accepted;
  }

  /**
   * Apply the change for a given file and update the editor with the result.
   *
   * @param {Change} change
   * @param {Editor} editor
   * @param {Blob} file
   * @return {Promise}
   */

  function asyncApplyChange(change, editor, file) {
    return _es6Promise2.default.resolve(insertImage(change, file, editor)).then(function () {
      editor.onChange(change);
    });
  }

  /**
   * On drop or paste.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   * @return {State}
   */

  function onInsert(event, change, editor) {
    var transfer = (0, _slateReact.getEventTransfer)(event);
    var range = (0, _slateReact.getEventRange)(event, change.value);
    switch (transfer.type) {
      case 'files':
        return onInsertFiles(event, change, editor, transfer, range);
      case 'html':
        return onInsertHtml(event, change, editor, transfer, range);
      case 'text':
        return onInsertText(event, change, editor, transfer, range);
    }
  }

  /**
   * On drop or paste files.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   * @param {Object} transfer
   * @param {Range} range
   * @return {Boolean}
   */

  function onInsertFiles(event, change, editor, transfer, range) {
    var files = transfer.files;
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {

      for (var _iterator2 = files[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        var file = _step2.value;

        if (extensions) {
          var type = file.type;

          var _type$split = type.split('/'),
              _type$split2 = _slicedToArray(_type$split, 2),
              ext = _type$split2[1];

          if (!matchExt(ext)) continue;
        }

        if (range) {
          change.select(range);
        }

        asyncApplyChange(change, editor, file);
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }

    return true;
  }

  /**
   * On drop or paste html.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   * @param {Object} transfer
   * @param {Range} range
   * @return {Boolean}
   */

  function onInsertHtml(event, change, editor, transfer, range) {
    var html = transfer.html;

    var parser = new DOMParser();
    var doc = parser.parseFromString(html, 'text/html');
    var body = doc.body;
    var firstChild = body.firstChild;
    if (firstChild.nodeName.toLowerCase() != 'img') return;

    var src = firstChild.src;

    if (extensions) {
      var ext = (0, _path.extname)(src).slice(1);
      if (!matchExt(ext)) return;
    }

    (0, _loadImageFile2.default)(src, function (err, file) {
      if (err) return;
      var c = editor.value.change();
      if (range) c.select(range);
      asyncApplyChange(c, editor, file);
    });

    return true;
  }

  /**
   * On drop or paste text.
   *
   * @param {Event} event
   * @param {Change} change
   * @param {Editor} editor
   * @param {Object} transfer
   * @param {Range} range
   * @return {Boolean}
   */

  function onInsertText(event, change, editor, transfer, range) {
    var text = transfer.text;

    if (!(0, _isUrl2.default)(text)) return;
    if (!(0, _isImage2.default)(text)) return;

    (0, _loadImageFile2.default)(text, function (err, file) {
      if (err) return;
      var c = editor.value.change();
      if (range) c.select(range);
      asyncApplyChange(c, editor, file);
    });

    return true;
  }

  /**
   * Return the plugin.
   *
   * @type {Object}
   */

  return {
    onDrop: onInsert,
    onPaste: onInsert
  };
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = DropOrPasteImages;